#!/usr/bin/env python3
# Test

import yaml
import sys
import os
import argparse
import os.path
import subprocess
import pkg_resources

from restafari import comparer
from restafari import connection
from restafari import output

conf = {
    'domain': '127.0.0.1',
    'protocol': 'http',
    'port': '80',
    'path': '',
    'cookies': {},
    'headers': {},
    'db': {'#exec': []},
    'errors': []
}

api_data = {
    '@req': {},
    '@res': {},
    '@hdr': {}
}


def loadFile(filename):
    global conf
    global api_data

    db = conf['db']
    if not os.path.exists(filename):
        filename = os.getcwd() + '/' + filename
    if not os.path.exists(filename):
        print('File ' + filename + ' does not exist.')
        sys.exit(1)
    print('[' + filename + ']')
    stream = open(filename, 'r')

    # Load YAML file
    try:
      obj = yaml.load(stream)
    except yaml.YAMLError as exc:
      print("The file \"" + filename +
            "\" is a malformed YAML. Please fix it before " +
            "executing it again:\n\n" + str(exc))
      print('-------------------')
      print('Here is a simple sample of a valid rest file:')
      valid_sample()
      sys.exit(1)

    for test in obj['tests']:
        id = test['id']
        if id in db:
            output.printLoadError(filename, id, "Duplicated id")
        db[id] = test

        if 'desc' not in test or test['desc'] is None:
            test['desc'] = "Test id: " + test['id']

        if 'method' not in test or test['method'] is None:
            test['method'] = 'GET'

        if 'header' not in test or test['header'] is None:
            test['header'] = {}
            test['header']['Content-type'] = 'application/json'

        if 'executable' not in test or test['executable'] is None:
            test['executable'] = True

        if test['executable'] is True:
            db["#exec"].append(id)

        print(output.load_color.format(db[id]['desc']))


def checkDeps():
    global conf
    db = conf['db']
    for test in db:
        if test[0] != "#":
            if 'expect' not in db[test]:
                output.printDepError(db[test]['id'], '', 'expect')
            if 'exec_before' in db[test]:
                bef = db[test]['exec_before']
            else:
                bef = []
            if 'exec_after' in db[test]:
                aft = db[test]['exec_after']
            else:
                aft = []
            if len(bef) > 0:
                for id in bef:
                    if id not in db:
                        output.printDepError(db[test]['id'], id, 'exec_before')


def runTests():
    global conf
    db = conf['db']
    for id in db['#exec']:
        if 'exec_before' in db[id]:
            for bef in db[id]['exec_before']:
                runTest(conf, bef, db)

        runTest(conf, id, db)

        if 'exec_after' in db[id]:
            for aft in db[id]['exec_after']:
                runTest(conf, aft, db)


def runTest(conf, id, db):
    global api_data
    req = connection.getRequest(id, conf, api_data)
    res = comparer.compareResult(req, db[id]['expect'], conf)

    if res is False:
        output.validationError(conf)
    else:
        output.validationOk(conf)


def main():
    global conf

    parser = argparse.ArgumentParser(
        description='Restafari REST API tester')
    parser.add_argument('--hostname', '-s',
                        dest='hostname',
                        help='The hostname of the web service')
    parser.add_argument('--protocol', '-p',
                        dest='protocol',
                        help='The protocol which will be used (HTTP/HTTPS)')
    parser.add_argument('--port', '-P', dest='port', help='The port number')
    parser.add_argument('--exec-before', '-B', dest='exec_before',
                        help='Execute a single command-line/script before ' +
                             'starting the REST test. It\'s useful ' +
                             'to prepare the environment for the tests.')
    parser.add_argument('--exec-success', '-S', dest='exec_success',
                        help='Execute a single command-line/script when all ' +
                             'tests were done successfully.')
    parser.add_argument('--debug', '-D', dest='debug',
                        help='Enable debug mode')
    parser.add_argument('files', nargs="*", metavar='N',
                        help='The test file(s) to be used')

    parser.add_argument('--version', '-v', action='store_true',
                        help='Show version number')

    # TODO
    # parser.add_argument('--format', '-f', dest='format',
    #   help='Set the test file format: YAML (default) or JSON')
    # parser.add_argument('--directory', '-d', dest='directory',
    #   help='Load all test files (file.rest) from a directory, ' +
    #   'skipping files which initiates with .')
    # parser.add_argument('--exec-failure', '-F', dest='exec_failure',
    #   help='Execute a command-line when some test fails.')
    # parser.add_argument('--restful', '-r', dest='restful',
    #   help='Execute tests with REST-compliant mode on')

    args = parser.parse_args()

    if args.version:
        show_version()

    if len(args.files) == 0:
        parser.print_help()
        sys.exit(1)

    if args.hostname:
        conf['domain'] = args.hostname
    if args.protocol:
        conf['protocol'] = args.protocol
    if args.port:
        conf['port'] = args.port

    print(output.step_color.format("Loading files"))

    for file in args.files:
        loadFile(file)
    if args.exec_before:
        print('Executing: [' + args.exec_before + ']')
        res = subprocess.call(args.exec_before, shell=True)
        if res > 0:
            print('--exec-before command returned: ' + str(res))
            sys.exit(1)

    print(output.step_color.format("Checking test dependencies"))

    checkDeps()
    runTests()
    if args.exec_success:
        print('Executing: [' + args.exec_success + ']')
        res = subprocess.call(args.exec_success, shell=True)
        if res > 0:
            print('--exec-success command returned: ' + str(res))
            sys.exit(1)
    if 'has_errors' in conf:
        sys.exit(1)


def show_version():
    print('restafari ' + pkg_resources.get_distribution("restafari").version)
    sys.exit(0)


def valid_sample():
    print("tests:\n" +
          "- id : mytest\n" +
          "  desc : \"This is my test\"\n" +
          "  path :  \"/url-path\"\n" +
          "  method : GET\n" +
          "  expect:\n" +
          "    http: 200\n" +
          "    data:\n" +
          "      ok : true\n")


def debug():
    os.environ.pop("DEBUG", 1)

if __name__ == "__main__":
    main()



from restafari import output
import json
import sys
import urllib.parse
import http.client
import socket
import os
import pystache


def getRequest(id, conf, api_data):

    db = conf['db']
    test = db[id]
    debug = 0
    if 'DEBUG' in os.environ and os.environ['DEBUG'] != '1':
        debug = 1

    if 'header' not in conf or conf['header'] is None:
        conf['header'] = {}

    headers = conf['header'].copy()
    if 'header' in db[id]:
        headers.update(db[id]['header'])

    method = test['method'].upper()

    test['path'] = pystache.render(test['path'], api_data)

    fullpath = conf['path'] + test['path']
    desc = test['desc']
    params = ''
    server = conf['domain'] + ':' + conf['port']
    try:
        conn = http.client.HTTPConnection(server)
    except IOError as err:
        conf['errors'].append("Server " + server + " not found!")
        output.validationError(conf)
        sys.exit(1)

    if 'data' not in test or test['data'] is None:
        test['data'] = {}

    api_data['@req'][id] = test['data']

    res = None
    params = ""

    try:
        if method == 'GET':
            res = conn.request(method, fullpath, None, headers)
        else:
            if debug == 1:
                params = json.dumps(test['data'], sort_keys=True, indent=4)
            else:
                params = json.dumps(test['data'])
            params = pystache.render(params, api_data)
            res = conn.request(method, fullpath, params, headers)

    except ConnectionRefusedError as exc:
        print("The hostname/port is reachable. Please check it before " +
              "executing it again: " + str(exc))
        sys.exit(1)
    except socket.gaierror as exc:
        print("The hostname/port is reachable. Please check it before " +
              "executing it again: " + str(exc))
        sys.exit(1)

    try:
        res = conn.getresponse()

    except http.client.HTTPException as exc:
        print("The hostname/port is reachable. Please check it before " +
              "executing it again: " + str(exc))
        sys.exit(1)

    data = res.read().decode("utf-8").strip()

    if len(data) > 0 and debug == 0 and data[0] == '{':
        received_data = json.dumps(json.loads(data), sort_keys=True, indent=4)
    else:
        if len(data) > 60:
            received_data = data.replace("\n", '')
            received_data = received_data[0:60] + '...'
        else:
            received_data = data

    print("\n" + output.test_sep_color.format("#" * 80))
    print(output.step_color.format(id))
    print(output.test_sep_color.format("#" * 80))

    if len(params) > 0 and debug == 0 and params[0] == '{':
        params = json.dumps(json.loads(params), sort_keys=True, indent=4)

    output.printRequest(desc,
                        conf['domain'],
                        fullpath,
                        method,
                        params,
                        received_data,
                        res.status)

    result = {}
    result['status'] = res.status
    result['header'] = res.getheaders()
    api_data['@hdr'][id] = result['header']

    for header in result['header']:
        print(header)
        # db[id]['header'][hname] = hvalue

    try:
        if len(data) > 0:
            result['data'] = json.loads(data)
        else:
            result['data'] = {}
        api_data['@res'][id] = result['data']

    except ValueError:
        print("Invalid JSON outout: ")
        if debug == 1:
            print(data)
    # finally:
    #   result['data'] = None

    print(output.token_color.format(json.dumps(api_data,
                                               sort_keys=True,
                                               indent=4)))

    return result



from colorclass import Color
import sys
import os

step_color = Color("{green}#### Starting{/green} " +
                   "\"{white}{0}{/white}\" {green}test{/green}")
load_color = Color("-- Loading {green}{0}" +
                   "{/green} {white}test{/white}")

test_sep_color = Color("{green}{0}{/green}")

token_color = Color("{white} ==== Avaliable tokens " +
                    "after this test:{/white}\n {blue}{0}{/blue}")

http_ok_color = Color("HTTP Status code:{green} [{0}] - OK{/green}")
http_notbad_color = Color("HTTP Status code:{yellow} [{0}]{/yellow}")
http_error_color = Color("HTTP Status code:{red} [{0}]{/red}")
http_request_color = Color("-- {white}{0} request ({1},{2},{3}){/white}" +
                           "    {white}\n ---> Request " +
                           "data: {magenta}{4}{/magenta}" +
                           "    {white}\n <--- Response " +
                           "data: {cyan}{5}{/cyan}")
load_error_color = Color("-- {red}Error on loading{/red} {white}{0}" +
                         "::id[{1}]: {2}{/white}")

dep_error_color = Color("-- {red}Dependecy Error:{/red} {white} from id::" +
                        "{0}; {1}->{/white}{yellow}{2}" +
                        "{/yellow}: {white}id does not exist!{/white}")

dep_color = Color("-- {white}Checking{/white} {green}{0}" +
                  "{/green} {white}test{/white}")

val_error_color = Color("{red} [FAIL] {/red} Validation error: {0}")
val_ok_color = Color("{green} [PASS] {/green} Validation passed")
verify_node_color = Color("{white} [TEST] {0}{/white} {1}")

invalid_op_color = Color("{yellow} [INVL] invalid operator {red}{0}" +
                         "{/red}{/yellow}: {1}")

invalid_exp_data_key_color = Color("invalid expected data key {yellow}{0}" +
                                   "{/yellow}. They is no property with this" +
                                   " name in server's response.")

debug_color = Color("{yellow} [DEBUG] {/yellow} {0}")


def printRequest(desc, domain, path, method, sent_data,
                 received_data, http_status_num):
    if http_status_num == 200:
        httpno = print(http_ok_color.format(str(http_status_num)))
    elif http_status_num > 200 and http_status_num < 400:
        httpno = print(http_notbad_color.format(str(http_status_num)))
    else:
        httpno = print(http_error_color.format(str(http_status_num)))
    if sent_data is "":
        sent_data = '<empty>'

    print(http_request_color.format(desc,
                                    domain,
                                    method,
                                    path,
                                    sent_data,
                                    received_data,
                                    httpno
                                    ))


def printLoadError(file, id, msg, errno=1):
    print(load_error_color.format(file, id, msg, errno))
    sys.exit(errno)


def printDepError(id_orig, id_missing, execx, errno=1):
    print(dep_error_color.format(id_orig, id_missing, execx))
    sys.exit(errno)


def printDeps(desc):
    print(dep_color.format(desc))


def validationError(conf):
    errors = conf['errors']
    for msg in errors:
        print(val_error_color.format(msg))
    conf['errors'] = []
    conf['has_errors'] = True


def validationOk(conf):
    print(val_ok_color)


def verifyNode(namespace, msg):
    print(verify_node_color.format(namespace, msg))


def invalidOperator(op, msg, conf):
    conf['errors'].append(invalid_op_color.format(op, msg))


def invalidExpectedDataKey(key, conf):
    conf['errors'].append(invalid_exp_data_key_color.format(key))


def debug(msg):
    if "DEBUG" in os.environ:
        print(debug_color.format(msg))



from restafari import output
import json
import sys


def compareResult(req, expect, conf):
    db = conf['db']

    if 'data' not in req:
        print("There is not expect.data from URL.")
        sys.exit(1)

    data = req['data']
    for key in expect.keys():
        # check value, if it has $, we are dealing with a compare
        # operator and need to use a function
        if key == 'data':
           for entity in expect['data']:
              ent = {}
              ent[entity] = expect['data'][entity]
              loadStructure(data, ent, conf, [])

        elif key == 'http':
            if req['status'] != expect['http']:
                msg = "Wrong HTTP status: expected {white}" + \
                      str(expect['http']) + '{/white}, got {yellow}' + \
                      str(req['status']) + '{/yellow}'
                conf['errors'].append(msg)

    return len(conf['errors']) == 0


def loadStructure(data, expect, conf, namespace):
    output.debug("Loading structure")
    result = True
    if type(expect) == dict and type(data) == dict:
        output.debug("Recursive Loading")
        for key in expect:
            if key[0] != '$' and key in data:
                return loadStructure(
                    data[key],
                    expect[key],
                    conf,
                    namespace + [key])

            elif checkOperator(key) and key in data:
                msg = str(data[key]) + key + '{gray} (from API){/gray}' + \
                    ' ' + str(expect[key]) + ' (expect value)'
                output.verifyNode(namespace, 'Comparing ' + key + ": " + msg)
                res = compare(data[key], key, expect[key])
                if res is False:
                    conf['errors'].append("Comparison failed: " + msg)
                result = res

            else:
                output.invalidExpectedDataKey(key, conf)
                return False
    elif type(expect) == dict and type(data) != dict:
        output.debug("Checking rules directly")
        for key in expect:
            if checkOperator(key):
                msg = '({magenta}' + "Value" + '{/magenta} {green}' + \
                      key + '{/green} ' + '{magenta}' + \
                      str(expect[key]) + '{/magenta}) ->'

                ops = []
                if expect[key] == dict:
                    for item in expect[key].keys():
                        ops.append(getOperatorDesc(item) + ' ' +
                                   str(expect[key][item]))
                    strops = ' {cyan}' + getOperatorDesc(key) + '{/cyan} be '
                else:
                    ops.append(str(expect[key]))
                    strops = ' {cyan}' + getOperatorDesc(key) + '{/cyan} be '

                    msg = msg + ' {black}(From API){/black} ' + \
                        ' {cyan}' + 'the value must be{/cyan} ' + \
                        strops.join(ops) + '{black} (expect value){/black} '

                    output.verifyNode(namespace, 'Comparing ' + msg)
                    res = compare(data, key, expect[key])
                    if res is False:
                        conf['errors'].append("Comparison failed: [" +
                                              json.dumps(namespace) +
                                              "] " + msg)
                    result = res
            else:
                output.invalidOperator(key, "the key was not found on " +
                                       "server's response", conf)
                return False
    elif type(expect) != dict and type(data) != list:
        output.debug("Straight comparison: " + str(expect) +
                     ' == ' + str(data))
        output.verifyNode(namespace, 'test')
        result = (expect == data)

    elif type(expect) != dict and type(data) != list and type(data) != dict:
        output.debug("Straight comparison: " + str(expect) +
                     ' == ' + str(data))
        output.verifyNode(namespace, 'test')
        result = (expect == data)
    return result


def checkOperator(op):
    return op in getOperators().keys()


def getOperatorDesc(op):
    return getOperators()[op]


def getOperators():
    return {
        '$gt': 'greater than',
        '$gte': 'greater than or equal',
        '$lt': 'lesser than',
        '$lte': 'lesser than or equal',
        '$eq': 'equal',
        '$ne': 'not equal',
        '$in': 'in',
        '$nin': 'not in',
        '$ignore': 'ignore',
        '$type': 'a',
        '$and': 'and',
        '$or': 'or',
        '$match': 'match',
        '$nullable': 'can be null',
        '$maxlen': 'maximum length of',
        '$minlen': 'minimum length of'
    }


def compare(value, op, exp):
    if op == '$or':
        return statementOr(value, exp)
    if op == '$and':
        return statementAnd(value, exp)
    if op == '$gt':
        return value > exp
    if op == '$gte':
        return value >= exp
    if op == '$lte':
        return value <= exp
    if op == '$lt':
        return value < exp
    if op == '$eq':
        return value == exp
    if op == '$ne':
        return value != exp
    if op == '$in':
        return value in exp
    if op == '$nin':
        return value not in exp
    if op == '$ignore':
        return exp is True
    if op == '$nullable':
        return exp is True and value is None
    if op == '$match':
        re.compile(exp)
        return prog.match(value)
    if op == '$maxlen':
        return len(value) <= exp
    if op == '$minlen':
        return len(value) >= exp
    if op == '$nullable':
        return exp is True and value is None
    if op == '$type':
        if type(value) is str and exp.upper() == 'STRING':
            return True
        if type(value) is int and exp.upper() == 'NUMBER':
            return True
        if type(value) is float and exp.upper() == 'NUMBER':
            return True
        if type(value) is bool and exp.upper() == 'BOOLEAN':
            return True
        if type(value) is None and exp.upper() == 'NULL':
            return True
        if type(value) is dict and exp.upper() == 'OBJECT':
            return True
        if type(value) is list and exp.upper() == 'ARRAY':
            return True

    return False


def statementOr(value, exp):
    if type(exp) != dict:
        return False
    for op in exp.keys():
        if compare(value, op, exp[op]):
            return True
    return False


def statementAnd(value, exp):
    if type(exp) != dict:
        return False
    for op in exp.keys():
        if not compare(value, op, exp[op]):
          return False
    return True



#!/usr/bin/env python3
from distutils.core import setup

version = "0.1.10"

setup(
    name = 'restafari',
    packages = ['restafari'], # this must be the same as the name above
    license = 'MIT',
    version = version,
    description = 'Restafari is a simple REST test tool.',
    author = 'Manoel Carvalho',
    author_email = 'manoelhc@gmail.com',
    url = 'https://github.com/manoelhc/restafari', # use the URL to the github repo
    download_url = 'https://github.com/manoelhc/restafari', # I'll explain this in a second
    keywords = ['testing', 'logging', 'rest', 'http', 'json'], # arbitrary keywords
    install_requires=[
        'anyjson',
        'PyYAML',
        'ColorClass',
        'cherrypy',
        'setuptools',
    ],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    entry_points={
        'console_scripts': [
            'restafari=restafari:main',
        ],
    }
)
